@@ -46,6 +46,7 @@ import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.cos.COSInteger;
 import org.apache.pdfbox.cos.COSName;
+import org.apache.pdfbox.io.RandomAccessReadBuffer;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDPage;
 import org.apache.pdfbox.pdmodel.PDResources;
@@ -166,12 +167,14 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
                 if (rotation == 90 || rotation == 270) {
                     at.scale(area.getWidth() / area.getHeight(), area.getHeight() / area.getWidth());
                 }
-                if (g2d instanceof PSGraphics2D && new PageUtil().pageHasTransparency(page.getResources(), page)) {
+                PageUtil pageUtil = new PageUtil();
+                if (g2d instanceof PSGraphics2D && pageUtil.pageHasTransparency(page.getResources(), page)) {
                     drawPageAsImage(at, g2d);
                 } else {
                     at.translate(area.getX(), area.getY());
                     at.scale(area.getWidth() / mediaBox.getWidth(),
                             area.getHeight() / mediaBox.getHeight());
+                    scaleToDPI(g2d, at, pageUtil);
                     g2d.transform(at);
                     normaliseScale(g2d);
                     PDFRenderer pdfRenderer = new PDFRenderer(pdDocument);
@@ -195,6 +198,16 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
             return r;
         }
 
+        private void scaleToDPI(Graphics2D g2d, AffineTransform at, PageUtil pageUtil) throws IOException {
+            if (g2d instanceof PSGraphics2D && pageUtil.hasTransparencyGroup) {
+                PSGraphics2D psGraphics2D = (PSGraphics2D) g2d;
+                double scaleDown = 72 / dpi;
+                psGraphics2D.getPSGenerator().concatMatrix(AffineTransform.getScaleInstance(scaleDown, scaleDown));
+                double scaleUp = dpi / 72d;
+                at.scale(scaleUp, scaleUp);
+            }
+        }
+
         private void normaliseScale(Graphics2D g2d) {
             if (!(g2d instanceof AbstractGraphics2D)) {
                 AffineTransform old = g2d.getTransform();
@@ -216,6 +229,7 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
         static class PageUtil {
             private List<COSDictionary> visited = new ArrayList<COSDictionary>();
             private Map<String, PDXObject> visitedXOjects = new HashMap<String, PDXObject>();
+            private boolean hasTransparencyGroup;
 
             private boolean pageHasTransparency(PDResources res, final PDPage page) throws IOException {
                 if (res != null) {
@@ -242,6 +256,7 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
 //                        }
                         }
                     }
+                    int count = 0;
                     for (COSName pdxObjectName : res.getXObjectNames()) {
                         PDXObject pdxObject = res.getXObject(pdxObjectName);
                         visitedXOjects.put(pdxObjectName.getName(), pdxObject);
@@ -249,7 +264,10 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
                             PDFormXObject form = (PDFormXObject) pdxObject;
                             if (form.getGroup() != null && COSName.TRANSPARENCY.equals(
                                     form.getGroup().getCOSObject().getDictionaryObject(COSName.S))) {
-                                return true;
+                                if (form.getGroup().getCOSObject().getItem(COSName.TYPE) != COSName.GROUP) {
+                                    return true;
+                                }
+                                count++;
                             }
                             PDResources formRes = form.getResources();
                             if (formRes != null && !visited.contains(formRes.getCOSObject())
@@ -258,6 +276,10 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
                             }
                         }
                     }
+                    hasTransparencyGroup = count > 0;
+                    if (count > 1) {
+                        return true;
+                    }
                 }
                 CheckImageMask checkImageMask = new CheckImageMask(visitedXOjects, page);
                 return checkImageMask.foundWhite;
@@ -275,8 +297,8 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
     }
 
     static final class CheckImageMask extends PDFStreamEngine {
-        private static final String DRAWOBJECT = new DrawObject().getName();
-        private static final String SETNONSTROKINGDEVICEGRAYCOLOR = new SetNonStrokingDeviceGrayColor().getName();
+        private static final String DRAWOBJECT = new DrawObject(null).getName();
+        private static final String SETNONSTROKINGDEVICEGRAYCOLOR = new SetNonStrokingDeviceGrayColor(null).getName();
         private boolean foundWhite;
         private boolean checkColor;
         private Map<String, PDXObject> xobjects;
@@ -350,8 +372,8 @@ public class ImageConverterPDF2G2D extends AbstractImageConverter {
                 try {
                     CustomFont font = getFont(postScriptName);
                     if (font instanceof MultiByteFont && !((MultiByteFont)font).isOTFFile()) {
-                        TTFParser ttfParser = new TTFParser(false, true);
-                        TrueTypeFont ttf = ttfParser.parse(font.getInputStream());
+                        TTFParser ttfParser = new TTFParser(false);
+                        TrueTypeFont ttf = ttfParser.parse(new RandomAccessReadBuffer(font.getInputStream()));
                         ttFonts.put(postScriptName, ttf);
                     }
                 } catch (IOException e) {
