@@ -55,7 +55,7 @@ public class StructureTreeMerger {
     PDFDocument pdfDoc;
     private PDPage srcPage;
     private COSDictionary roleMap;
-    protected PDFStructElem currentSessionElem;
+    private PDFStructElem currentSessionElem;
     private PDFLogicalStructureHandler logicalStructHandler;
     private Map<Integer, PDFStructElem> structElemCache = new HashMap<Integer, PDFStructElem>();
     private Map<Integer, PDFStructElem> markedContentMap = new TreeMap<Integer, PDFStructElem>();
@@ -114,24 +114,41 @@ public class StructureTreeMerger {
             copyElemEntries(obj, elem);
             parent.addKid(elem);
             elem.setParent(parent);
-            COSBase objKid = obj.getItem(COSName.K);
+            COSDictionary dict = (COSDictionary) obj.getObject();
+            COSBase objKid = dict.getItem(COSName.K);
             if (objKid != null) {
                 createDirectDescendants(objKid, elem);
             }
         }
     }
 
+    public PDFStructElem getCurrentSessionElem() {
+        return currentSessionElem;
+    }
+
     public void setCurrentSessionElem() {
-        if (currentSessionElem == null) {
-            currentSessionElem = pdfDoc.getStructureTreeElements()
-                    .get(pdfDoc.getStructureTreeElements().size() - 1);
+        if (isCurrentSessionElemEmpty(currentSessionElem)) {
+            List<PDFStructElem> structureTreeElements = pdfDoc.getStructureTreeElements();
+            for (int i = structureTreeElements.size() - 1; i >= 0; i--) {
+                PDFStructElem elem = structureTreeElements.get(i);
+                // We need to make sure it's not an empty element (like an empty block)
+                if (!isCurrentSessionElemEmpty(elem)) {
+                    currentSessionElem = elem;
+                    break;
+                }
+            }
         }
     }
 
+    private boolean isCurrentSessionElemEmpty(PDFStructElem elem) {
+        return elem == null || elem.get("P") == null;
+    }
+
     private void createParents(COSArray markedContentParents) throws IOException {
         for (COSBase entry : markedContentParents) {
             COSObject elemCos = (COSObject)entry;
-            COSObject elemParent = (COSObject)elemCos.getItem(COSName.P);
+            COSDictionary dict = (COSDictionary) elemCos.getObject();
+            COSObject elemParent = (COSObject)dict.getItem(COSName.P);
             if (elemParent != null) {
                 PDFStructElem elem = structElemCache.get((int)elemCos.getObjectNumber());
                 createParents(elemCos, elemParent, elem);
@@ -191,7 +208,8 @@ public class StructureTreeMerger {
                 int position = StructureTreeMergerUtil.findObjectPositionInKidsArray(cosElem);
                 elemParent.addKidInSpecificOrder(position, elem);
             }
-        } else if (!checkIfStructureTypeIsPresent(parentElemDictionary, StandardStructureTypes.DOCUMENT)) {
+        } else if (!(checkIfStructureTypeIsPresent(parentElemDictionary, StandardStructureTypes.DOCUMENT)
+                || checkIfStructureTypeIsPresent(parentElemDictionary, StandardStructureTypes.PART))) {
             elemParent = createAndRegisterStructElem(cosParentElem);
             copyElemEntries(cosParentElem, elemParent);
             elem.setParent(elemParent);
